// knihovna Simulation
// Poskytuje zakladni tridy pro diskretni simulaci
//////////////////////////////////////////////////////////////////////

#include "simset.h"
#include "eventnotice.h"
#include "thread.h"

#if !defined(PROCESS_H)
#define PROCESS_H

//////////////////////////////////////////////////////////////////////
// Process.h: interface for the CProcess class.
// popis procesu vcetne metod pro jeho planovani
// obdoba tridy process + planovaci metody tridy SIMULATION jazyka Simula67
//////////////////////////////////////////////////////////////////////

#define EVENT_IDLE -1	

class CEventNotice;
enum CODE {DIRECT, AT, DELAY, BEFORE, AFTER};
		// presnejsi specifikace planovaciho prikazu

 typedef double TIME;

/*******************************************************************/
// definice planovacich maker Activate a Reactivate

#define Activate() _activate(false, DIRECT, 0, NULL, false)
#define ActivateAt(t) _activate(false, AT, t, NULL, false)
#define ActivateAtPrior(t) _activate(false, AT, t, NULL, true)
#define ActivateDelay(t) _activate(false, DELAY, t, NULL, false)
#define ActivateDelayPrior(t) _activate(false, DELAY, t, NULL, true)
#define ActivateBefore(r) _activate(false, BEFORE, 0, r, false)
#define ActivateAfter(r) _activate(false, AFTER, 0, r, false)
#define Reactivate() _activate(true, DIRECT, 0, NULL, false)
#define ReactivateAt(t) _activate(true, AT, t, NULL, false)
#define ReactivateAtPrior(t) _activate(true, AT, t, NULL, true)
#define ReactivateDelay(t) _activate(true, DELAY, t, NULL, false)
#define ReactivateDelayPrior(t) _activate(true, DELAY, t, NULL, true)
#define ReactivateBefore(r) _activate(true, BEFORE, 0, r, false)
#define ReactivateAfter(r) _activate(true, AFTER, 0, r, false)

/*******************************************************************/

class CProcess : public CLink, public CThread
{

private: bool terminated;	// true pokud je proces ukonceny

public:
    static CHead* SQS;	  // seznam SQS spolecny pro vsechny procesy

	CEventNotice* event;  // odkaz na zaznam o udalosti tohoto procesu
	
	CProcess* NextEv();
	TIME EvTime();

	static CProcess* Current();
	static CEventNotice* FirstEv();
	static TIME Time();
	
	virtual void Run() = 0;		// touto metodou uzivatel popisuje
								// vlastni chovani procesu
	bool Idle();
	bool Terminated();
	
	void _activate(bool reac, CODE code, TIME t, CProcess* r, bool prior);
	void Cancel(CProcess* p);
	void Wait(CHead* S);
	void Passivate();
	void Hold(TIME T);
	
	CProcess(bool createThread = true);
	virtual ~CProcess();
	static void *iq(void *coze);// { (CProcess *) coze->Start();  }
private:
	virtual void Start();
	void ResumeCurrent();

};

#endif // !defined(PROCESS_H)
